import os, json
import streamlit.components.v1 as components

from pollination_streamlit.selectors import get_api_client

from pollination_streamlit_io import (create_study)

import streamlit as st

api_client = get_api_client()

create_study('foo', api_client)